@@ -1,9 +1,10 @@
 """
 Create a git commit message.
 """
+import os
 import re
-import textwrap
 from typing import Iterator
+from typing import Optional
 
 from git import Repo
 
@@ -14,6 +15,42 @@ from embedit.behaviour.openai_tools import tokclip
 from embedit.behaviour.openai_tools import toklen
 from embedit.behaviour.prompts.default import default_pre_prompt
 
+musings_on_good_vs_great_commit_messages = """
+A good commit message should:
+
+- Be written in the present tense
+- Be brief and to the point (less than 50 characters is a good rule of thumb for the subject line)
+- Describe the changes made in the commit
+A great commit message goes beyond that and also:
+
+- Explains the why behind the change, not just the what
+- Provides context for the change
+- Is written in a clear and concise manner
+- Includes any relevant issue or ticket numbers
+- Includes any testing done
+
+Writing clear and informative commit messages makes it easier for others to understand the changes made to the codebase and can help when debugging and reviewing pull requests.
+
+Good commit message:
+
+"Fix bug in login form"
+Great commit message:
+
+"Fix incorrect validation in login form #1234
+The form was failing to validate the password field correctly and throwing an error, now the validation has been updated to correctly match the password against the database. Tested on multiple scenarios and confirmed to work as expected."
+
+The "good" commit message is brief and clearly states the changes made, but it does not explain why the change was necessary or provide any context.
+
+The "great" commit message, on the other hand, not only states the change but also explains the reasoning behind it, provides context, and includes relevant information such as the issue number and testing done. This level of detail makes it easier for others to understand the change and its impact on the codebase.
+"""
+
+pre_prompt_commit = "\n\n".join(
+    [
+        default_pre_prompt,
+        musings_on_good_vs_great_commit_messages,
+    ]
+)
+
 
 def make_builtin_examples():
     return [
@@ -64,33 +101,22 @@ def make_builtin_examples():
     ]
 
 
-def make_prompt(quality: int) -> str:
-    return f"Write a commit message for the given diff (quality: {quality}/10)"
+def make_prompt(quality: int, hint: Optional[str] = None) -> str:
+    prompt = f"Write a commit message for the given diff (quality: {quality}/10)"
+    if hint is not None:
+        prompt += f" (hint: {hint})"
+    return prompt
 
 
 def rate_commit_message(commit_message: str) -> int:
     """
     Rate the given commit message on a scale of 0 to 10.
     """
-    pre_prompt = textwrap.dedent(
-        """
-            Comment on the following commit messages. Rate them on a scale of 0 to 10.
-            
-            A good commit message should:
-        
-            - Be written in the present tense
-            - Be brief and to the point (less than 50 characters is a good rule of thumb for the subject line)
-            - Describe the changes made in the commit
-            A great commit message goes beyond that and also:
-            
-            - Explains the why behind the change, not just the what
-            - Provides context for the change
-            - Is written in a clear and concise manner
-            - Includes any relevant issue or ticket numbers
-            - Includes any testing done
-            
-            Writing clear and informative commit messages makes it easier for others to understand the changes made to the codebase and can help when debugging and reviewing pull requests.
-            """
+    pre_prompt = "\n\n".join(
+        [
+            "Comment on the following commit messages. Rate them on a scale of 0 to 10.",
+            musings_on_good_vs_great_commit_messages,
+        ]
     )
 
     examples = [
@@ -166,8 +192,8 @@ def get_diffstrs(path: str, max_diff_tokens: int) -> Iterator[str]:
     If all the diffs together are within the max_diff_tokens limit, yield them all together. Otherwise, yield them one at a time.
     """
     repo = Repo(path)
-    # Diff the staged changes
-    diff = list(repo.index.diff("HEAD", create_patch=True))
+    # Diff between head and staged
+    diff = list(repo.index.diff(repo.head.commit, create_patch=True, R=True))
     if len(diff) == 0:
         raise ValueError("No changes to commit. Have you staged your changes?")
     diff_chunk = []
@@ -184,6 +210,8 @@ def get_diffstrs(path: str, max_diff_tokens: int) -> Iterator[str]:
             diff_chunk = []
             diff_chunk_size = 0
         diff_chunk.append(str(d))
+    if len(diff_chunk) > 0:
+        yield "\n".join(diff_chunk)
 
 
 def make_commit_message(
@@ -191,12 +219,16 @@ def make_commit_message(
     max_log_tokens: int = 1400,
     max_diff_tokens: int = 1400,
     max_output_tokens: int = 400,
-    engine: str = "text-davinci-003",
+    engine: str = "code-davinci-002",
     num_examples: int = 10,
     use_builtin_examples: bool = True,
+    hint: Optional[str] = None,
+    num_lines_context: int = 10,
     verbose: bool = False,
 ) -> str:
     """Return a commit message based on the given diff."""
+    # Set the GIT_DIFF_OPTS environment variable to change the number of lines of context shown in the diff.
+    os.environ["GIT_DIFF_OPTS"] = f"-u{num_lines_context}"
     diffstrs = list(get_diffstrs(path=path, max_diff_tokens=max_diff_tokens))
     if use_builtin_examples:
         examples = make_builtin_examples()
@@ -208,28 +240,28 @@ def make_commit_message(
     if len(diffstrs) == 1:
         return complete(
             context=diffstrs[0],
-            prompt=make_prompt(quality=10),
+            prompt=make_prompt(quality=10, hint=hint),
             examples=examples,
-            pre_prompt=default_pre_prompt,
+            pre_prompt=pre_prompt_commit,
             max_output_tokens=max_output_tokens,
             engine=engine,
             verbose=verbose,
         ).strip()
     else:
+        assert len(diffstrs) > 1
         # Process each diff separately and combine the results
         messages = []
         for diffstr in diffstrs:
             message = complete(
                 context=diffstr,
-                prompt=make_prompt(quality=10),
+                prompt=make_prompt(quality=10, hint=hint),
                 examples=examples,
-                pre_prompt=default_pre_prompt,
+                pre_prompt=pre_prompt_commit,
                 max_output_tokens=max_output_tokens,
                 engine=engine,
                 verbose=verbose,
             ).strip()
             messages.append(message)
-        assert len(messages) > 1
         combine_examples = [
             (
                 Task(
